require('reflect-metadata');
var Container = (function () {
    function Container() {
    }
    // -------------------------------------------------------------------------
    // Public Static Methods
    // -------------------------------------------------------------------------
    Container.registerCustomParamHandler = function (paramHandler) {
        this.customParamHandlers.push(paramHandler);
    };
    Container.registerDefaultInitializationParameter = function (type, params) {
        this.defaultParameters.push({ type: type, params: params });
    };
    Container.get = function (type, params) {
        var obj = this.findInstanceOfType(type);
        if (obj)
            return obj;
        if (!params) {
            var defaultParams = this.defaultParameters.reduce(function (found, i) { return i.type === type ? i : found; }, undefined);
            if (defaultParams)
                params = defaultParams.params;
        }
        if (params) {
            params = this.mapParams(type, params);
            params.unshift(null);
        }
        var objectInstance = new (type.bind.apply(type, params))();
        this.instances.push({ type: type, instance: objectInstance });
        return objectInstance;
    };
    Container.set = function (type, value) {
        this.instances.push({ type: type, instance: value });
    };
    // -------------------------------------------------------------------------
    // Private Static Methods
    // -------------------------------------------------------------------------
    Container.findInstanceOfType = function (type) {
        return this.instances.reduce(function (found, typeInstance) { return typeInstance.type === type ? typeInstance.instance : found; }, null);
    };
    Container.findCustomParamHandler = function (type, index) {
        return this.customParamHandlers.reduce(function (found, param) { return param.type === type && param.index === index ? param : found; }, null);
    };
    Container.mapParams = function (type, params) {
        var _this = this;
        return params.map(function (param, key) {
            var paramHandler = _this.findCustomParamHandler(type, key);
            if (paramHandler)
                return paramHandler.getValue();
            if (_this.isParamValid(param))
                return Container.get(param);
            return undefined;
        });
    };
    Container.isParamValid = function (param) {
        var ignoredTypes = ['string', 'boolean', 'number', 'object'];
        if (param && param.name && ignoredTypes.indexOf(param.name.toLowerCase()) !== -1)
            return false;
        return true;
    };
    // -------------------------------------------------------------------------
    // Static Properties
    // -------------------------------------------------------------------------
    Container.instances = [];
    Container.customParamHandlers = [];
    Container.defaultParameters = [];
    return Container;
})();
exports.Container = Container;
